@@ -2,19 +2,23 @@
 **  Copyright (c) 2006, 2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: config.h,v 1.2 2007/03/05 20:52:53 msk Exp $
+**  $Id: config.h,v 1.3 2007/08/03 15:28:10 msk Exp $
 */
 
 #ifndef _CONFIG_H_
 #define _CONFIG_H_
 
 #ifndef lint
-static char config_h_id[] = "@(#)$Id: config.h,v 1.2 2007/03/05 20:52:53 msk Exp $";
+static char config_h_id[] = "@(#)$Id: config.h,v 1.3 2007/08/03 15:28:10 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
+#include <sys/types.h>
 #include <stdio.h>
 
+/* libsm includes */
+#include <sm/gen.h>
+
 /* dkim-filter includes */
 #include "dkim-filter.h"
 
@@ -25,9 +29,11 @@ static char config_h_id[] = "@(#)$Id: config.h,v 1.2 2007/03/05 20:52:53 msk Exp
 
 struct config
 {
+	bool		cfg_bool;
 	u_int		cfg_type;
+	int		cfg_int;
 	char *		cfg_name;
-	void *		cfg_data;
+	char *		cfg_string;
 	struct config *	cfg_next;
 };
 
@@ -41,7 +47,7 @@ struct configdef
 /* prototypes */
 extern char *config_check __P((struct config *, struct configdef *));
 extern void config_free __P((struct config *));
-extern int config_get __P((struct config *, const char *, void *));
+extern int config_get __P((struct config *, const char *, void *, size_t));
 extern struct config *config_load __P((FILE *, struct configdef *,
                                        unsigned int *));
 
