@@ -2,18 +2,18 @@
 **  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-filter.h,v 1.71 2007/06/26 16:26:46 msk Exp $
+**  $Id: dkim-filter.h,v 1.94 2007/08/22 08:45:10 msk Exp $
 */
 
 #ifndef _DKIM_FILTER_H_
 #define _DKIM_FILTER_H_
 
 #ifndef lint
-static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.71 2007/06/26 16:26:46 msk Exp $";
+static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.94 2007/08/22 08:45:10 msk Exp $";
 #endif /* !lint */
 
 #define	DKIMF_PRODUCT	"Sendmail DKIM Filter"
-#define	DKIMF_VERSION	"1.2.0"
+#define	DKIMF_VERSION	"2.1.2"
 
 /* libsm */
 #include <sm/gen.h>
@@ -30,17 +30,17 @@ static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.71 2007/06/26 16:26
 #endif /* !TRUE */
 
 /* defaults, limits, etc. */
-#define	BUFRSZ		256
+#define	BUFRSZ		1024
 #define	CBINTERVAL	3
-#define CMDLINEOPTS	"a:Ab:c:C:d:DfFhi:I:k:KlL:m:M:o:p:P:qrRs:S:T:u:U:Vx:?"
+#define CMDLINEOPTS	"a:Ab:c:C:d:Dfhi:I:k:KlL:m:M:o:p:P:qrRs:S:T:u:U:Vx:?"
 #define	DEFTIMEOUT	5
 #define	HOSTUNKNOWN	"unknown-host"
 #define	JOBIDUNKNOWN	"(unknown-jobid)"
 #define	LOCALHOST	"127.0.0.1"
 #define	MAXADDRESS	256
 #define	MAXARGV		65536
+#define	MAXBUFRSZ	65536
 #define	MAXHDRLEN	78
-#define	MAXHEADER	4096
 #define	MAXSIGNATURE	1024
 #define	TEMPFILE	"/var/tmp/dkXXXXXX"
 #define	UNKNOWN		"unknown"
@@ -50,6 +50,11 @@ static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.71 2007/06/26 16:26
 #define	XHEADERNAME	"X-DKIM"
 #define	XSELECTCANONHDR	"X-Canonicalization"
 
+#ifdef _FFR_VBR
+# define XVBRTYPEHEADER	"X-VBR-Type"
+# define XVBRCERTHEADER	"X-VBR-Certifiers"
+#endif /* _FFR_VBR */
+
 /* POPAUTH db */
 #if POPAUTH
 # define POPAUTHDB	"/etc/mail/popip.db"
