@@ -2,14 +2,14 @@
 **  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: stats.h,v 1.3 2007/05/18 17:48:44 msk Exp $
+**  $Id: stats.h,v 1.4 2007/06/27 19:49:23 msk Exp $
 */
 
 #ifndef _STATS_H_
 #define _STATS_H_
 
 #ifndef lint
-static char stats_h_id[] = "@(#)$Id: stats.h,v 1.3 2007/05/18 17:48:44 msk Exp $";
+static char stats_h_id[] = "@(#)$Id: stats.h,v 1.4 2007/06/27 19:49:23 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -41,7 +41,6 @@ struct dkim_stats_key
 struct dkim_stats_data
 {
 	bool		sd_lengths;
-	int		sd_lastversion;
 	time_t		sd_lastseen;
 	dkim_alg_t	sd_lastalg;
 	u_long		sd_pass;
@@ -51,7 +50,7 @@ struct dkim_stats_data
 /* PROTOTYPES */
 extern void dkimf_stats_init __P((void));
 extern void dkimf_stats_record __P((const char *, const char *, dkim_canon_t,
-                                    dkim_canon_t, dkim_alg_t, int, bool, bool,
+                                    dkim_canon_t, dkim_alg_t, bool, bool,
                                     bool));
 
 #endif /* _STATS_H_ */
