@@ -2,11 +2,11 @@
 **  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: util.c,v 1.35 2007/06/25 20:30:36 msk Exp $
+**  $Id: util.c,v 1.41 2007/08/15 01:10:44 msk Exp $
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.35 2007/06/25 20:30:36 msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.41 2007/08/15 01:10:44 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -71,14 +71,6 @@ static char *optlist[] =
 	"_FFR_CAPTURE_UNKNOWN_ERRORS",
 #endif /* _FFR_CAPTURE_UNKNOWN_ERRORS */
 
-#if _FFR_FLUSH_HEADERS
-	"_FFR_FLUSH_HEADERS",
-#endif /* _FFR_FLUSH_HEADERS */
-
-#if _FFR_HASH_BUFFERING
-	"_FFR_HASH_BUFFERING",
-#endif /* _FFR_HASH_BUFFERING */
-
 #if _FFR_QUERY_CACHE
 	"_FFR_QUERY_CACHE",
 #endif /* _FFR_QUERY_CACHE */
@@ -170,40 +162,6 @@ dkimf_setmaxfd(void)
 	}
 }
 
-/*
-**  DKIMF_STRIPBRACKETS -- remove angle brackets from the sender address
-**
-**  Parameters:
-** 	addr -- address to be processed
-**
-**  Return value:
-**  	None.
-*/
-
-void
-dkimf_stripbrackets(char *addr)
-{
-	char *p, *q;
-
-	assert(addr != NULL);
-
-	p = addr;
-	q = addr + strlen(addr) - 1;
-
-	while (*p == '<' && *q == '>')
-	{
-		p++;
-		*q-- = '\0';
-	}
-
-	if (p != addr)
-	{
-		for (q = addr; *p != '\0'; p++, q++)
-			*q = *p;
-		*q = '\0';
-	}
-}
-
 /*
 **  DKIMF_LOWERCASE -- lowercase-ize a string
 **
@@ -753,3 +711,43 @@ dkimf_mkpath(char *path, size_t pathlen, char *root, char *file)
 		sm_strlcat(path, file, pathlen);
 	}
 }
+
+/*
+**  DKIMF_HOSTLIST -- see if a hostname is in a pattern of hosts/domains
+**
+**  Parameters:
+**  	host -- hostname to compare
+**   	list -- NULL-terminated char * array to search
+**
+**  Return value:
+**  	TRUE iff either "host" was in the list or it match a domain pattern
+**  	found in the list.
+*/
+
+bool
+dkimf_hostlist(char *host, char **list)
+{
+	int c;
+	char *p;
+
+	assert(host != NULL);
+	assert(list != NULL);
+
+	/* walk the entire list */
+	for (c = 0; list[c] != NULL; c++)
+	{
+		/* first try a full hostname match */
+		if (strcasecmp(host, list[c]) == 0)
+			return TRUE;
+
+		/* try each domain */
+		for (p = strchr(host, '.'); p != NULL; p = strchr(p + 1, '.'))
+		{
+			if (strcasecmp(p, list[c]) == 0)
+				return TRUE;
+		}
+	}
+
+	/* not found */
+	return FALSE;
+}
