@@ -2,14 +2,14 @@
 **  Copyright (c) 2004, 2005, 2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: util.h,v 1.9 2007/04/24 17:31:34 msk Exp $
+**  $Id: util.h,v 1.11 2007/08/15 01:10:44 msk Exp $
 */
 
 #ifndef _UTIL_H_
 #define _UTIL_H_
 
 #ifndef lint
-static char util_h_id[] = "@(#)$Id: util.h,v 1.9 2007/04/24 17:31:34 msk Exp $";
+static char util_h_id[] = "@(#)$Id: util.h,v 1.11 2007/08/15 01:10:44 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -35,6 +35,7 @@ extern bool dkimf_checkip __P((Peer, struct sockaddr *));
 #ifdef POPAUTH
 extern bool dkimf_checkpopauth __P((DB *, struct sockaddr *));
 #endif /* POPAUTH */
+extern bool dkimf_hostlist __P((char *, char **));
 extern size_t dkimf_inet_ntoa __P((struct in_addr, char *, size_t));
 #ifdef POPAUTH
 extern int dkimf_initpopauth __P((void));
@@ -44,7 +45,6 @@ extern void dkimf_lowercase __P((char *));
 extern void dkimf_mkpath __P((char *, size_t, char *, char *));
 extern void dkimf_optlist __P((FILE *));
 extern void dkimf_setmaxfd __P((void));
-extern void dkimf_stripbrackets __P((char *));
 extern void dkimf_stripcr __P((char *));
 extern void dkimf_trimspaces __P((char *));
 
