@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.18 2007/05/21 22:19:33 msk Exp $";
+static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.22 2007/08/14 20:40:52 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -36,9 +36,6 @@ static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.18 2007/05/21 22:19:33
 /* prototypes */
 extern void dkim_error __P((DKIM *, const char *, ...));
 
-/* limits, macros, etc. */
-#define	BUFRSZ			1025
-
 /* local definitions needed for DNS queries */
 #ifndef MAXHOSTNAMELEN
 # define MAXHOSTNAMELEN		256
@@ -55,6 +52,7 @@ extern void dkim_error __P((DKIM *, const char *, ...));
 **
 **  Parameters:
 **  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
 **  	buf -- buffer into which to write the result
 **  	buflen -- bytes available at "buf"
 **
@@ -63,7 +61,7 @@ extern void dkim_error __P((DKIM *, const char *, ...));
 */
 
 DKIM_STAT
-dkim_get_key_dns(DKIM *dkim, u_char *buf, size_t buflen)
+dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 {
 #ifdef _FFR_QUERY_CACHE
 	bool cached = FALSE;
@@ -97,14 +95,15 @@ dkim_get_key_dns(DKIM *dkim, u_char *buf, size_t buflen)
 	HEADER hdr;
 
 	assert(dkim != NULL);
-	assert(dkim->dkim_selector != NULL);
-	assert(dkim->dkim_domain != NULL);
-	assert(dkim->dkim_querymethod == DKIM_QUERY_DNS);
+	assert(sig != NULL);
+	assert(sig->sig_selector != NULL);
+	assert(sig->sig_domain != NULL);
+	assert(sig->sig_query == DKIM_QUERY_DNS);
 
 	lib = dkim->dkim_libhandle;
 
-	snprintf(qname, sizeof qname - 1, "%s.%s.%s", dkim->dkim_selector,
-	         DKIM_DNSKEYNAME, dkim->dkim_domain);
+	snprintf(qname, sizeof qname - 1, "%s.%s.%s", sig->sig_selector,
+	         DKIM_DNSKEYNAME, sig->sig_domain);
 
 #ifdef _FFR_QUERY_CACHE
 	/* see if we have this data already cached */
@@ -127,6 +126,66 @@ dkim_get_key_dns(DKIM *dkim, u_char *buf, size_t buflen)
 #endif /* _FFR_QUERY_CACHE */
 
 #if USE_ARLIB
+# ifdef _FFR_DNS_UPGRADE
+	for (c = 0; c < 2; c++)
+	{
+		switch (c)
+		{
+		  case 0:
+			ar = dkim->dkim_libhandle->dkiml_arlib;
+			break;
+
+		  case 1:
+			ar = dkim->dkim_libhandle->dkiml_arlibtcp;
+			break;
+		}
+
+		timeout.tv_sec = dkim->dkim_timeout;
+		timeout.tv_usec = 0;
+
+		q = ar_addquery(ar, qname, C_IN, T_TXT, MAXCNAMEDEPTH, ansbuf,
+		                sizeof ansbuf, &error,
+		                dkim->dkim_timeout == 0 ? NULL : &timeout);
+		if (q == NULL)
+		{
+			dkim_error(dkim, "ar_addquery() for `%s' failed",
+			           qname);
+			return DKIM_STAT_INTERNAL;
+		}
+
+		if (lib->dkiml_dns_callback == NULL)
+		{
+			status = ar_waitreply(ar, q, NULL, NULL);
+		}
+		else
+		{
+			for (;;)
+			{
+				timeout.tv_sec = lib->dkiml_callback_int;
+				timeout.tv_usec = 0;
+
+				status = ar_waitreply(ar, q, NULL, &timeout);
+
+				if (status != AR_STAT_NOREPLY)
+					break;
+
+				lib->dkiml_dns_callback(dkim->dkim_user_context);
+			}
+		}
+
+		(void) ar_cancelquery(ar, q);
+
+		/* see if the UDP reply was truncated */
+		if (c == 0 && status == AR_STAT_SUCCESS)
+		{
+			memcpy(&hdr, ansbuf, sizeof hdr);
+			if (hdr.tc)
+				continue;
+		}
+
+		break;
+	}
+# else /* _FFR_DNS_UPGRADE */
 	ar = dkim->dkim_libhandle->dkiml_arlib;
 
 	timeout.tv_sec = dkim->dkim_timeout;
@@ -162,6 +221,7 @@ dkim_get_key_dns(DKIM *dkim, u_char *buf, size_t buflen)
 	}
 
 	(void) ar_cancelquery(ar, q);
+# endif /* _FFR_DNS_UPGRADE */
 #else /* USE_ARLIB */
 	status = res_query(qname, C_IN, T_TXT, ansbuf, sizeof ansbuf);
 #endif /* USE_ARLIB */
@@ -241,11 +301,6 @@ dkim_get_key_dns(DKIM *dkim, u_char *buf, size_t buflen)
 	if (hdr.rcode == NXDOMAIN)
 		return DKIM_STAT_NOKEY;
 
-	/* get the answer count */
-	ancount = ntohs((unsigned short) hdr.ancount);
-	if (ancount == 0)
-		return DKIM_STAT_CANTVRFY;
-
 	/* if truncated, we can't do it */
 	if (hdr.tc)
 	{
@@ -253,6 +308,11 @@ dkim_get_key_dns(DKIM *dkim, u_char *buf, size_t buflen)
 		return DKIM_STAT_INTERNAL;
 	}
 
+	/* get the answer count */
+	ancount = ntohs((unsigned short) hdr.ancount);
+	if (ancount == 0)
+		return DKIM_STAT_CANTVRFY;
+
 	/*
 	**  Extract the data from the first answer.
 	*/
@@ -335,6 +395,7 @@ dkim_get_key_dns(DKIM *dkim, u_char *buf, size_t buflen)
 **
 **  Parameters:
 **  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
 **  	buf -- buffer into which to write the result
 **  	buflen -- bytes available at "buf"
 **
@@ -354,18 +415,19 @@ dkim_get_key_dns(DKIM *dkim, u_char *buf, size_t buflen)
 */
 
 DKIM_STAT
-dkim_get_key_file(DKIM *dkim, u_char *buf, size_t buflen)
+dkim_get_key_file(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 {
 	FILE *f;
-	char *p;
-	char *p2;
+	u_char *p;
+	u_char *p2;
 	char *path;
 	char name[BUFRSZ + 1];
 
 	assert(dkim != NULL);
-	assert(dkim->dkim_selector != NULL);
-	assert(dkim->dkim_domain != NULL);
-	assert(dkim->dkim_querymethod == DKIM_QUERY_FILE);
+	assert(sig != NULL);
+	assert(sig->sig_selector != NULL);
+	assert(sig->sig_domain != NULL);
+	assert(sig->sig_query == DKIM_QUERY_FILE);
 
 	path = dkim->dkim_libhandle->dkiml_queryinfo;
 	if (path[0] == '\0')
@@ -381,8 +443,8 @@ dkim_get_key_file(DKIM *dkim, u_char *buf, size_t buflen)
 		return DKIM_STAT_KEYFAIL;
 	}
 
-	snprintf(name, sizeof name, "%s.%s.%s", dkim->dkim_selector,
-	         DKIM_DNSKEYNAME, dkim->dkim_domain);
+	snprintf(name, sizeof name, "%s.%s.%s", sig->sig_selector,
+	         DKIM_DNSKEYNAME, sig->sig_domain);
 
 	memset(buf, '\0', sizeof buf);
 	while (fgets(buf, BUFRSZ, f) != NULL)
