@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.19 2007/06/25 20:24:47 msk Exp $";
+static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.26 2007/08/09 15:05:01 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -35,13 +35,13 @@ static struct chartable prv_keyparams[] =	/* key parameters */
 };
 struct chartable *keyparams = prv_keyparams;
 
-static struct chartable prv_pflags[] =		/* policy flags */
+static struct chartable prv_policyflags[] =	/* policy flags */
 {
 	{ 'y',		DKIM_PFLAG_TEST },
 	{ 's',		DKIM_PFLAG_NOSUBDOMAIN },
 	{ '\0',		-1 }
 };
-struct chartable *pflags = prv_pflags;
+struct chartable *policyflags = prv_policyflags;
 
 static struct nametable prv_policyparams[] =	/* policy parameters */
 {
@@ -60,6 +60,21 @@ static struct nametable prv_policies[] =	/* policies */
 };
 struct nametable *policies = prv_policies;
 
+static struct nametable prv_policyresults[] =	/* policy results */
+{
+	{ "none",				DKIM_PRESULT_NONE },
+	{ "domain does not exist",		DKIM_PRESULT_NXDOMAIN },
+	{ "policy \"unknown\"",			DKIM_PRESULT_UNKNOWN },
+	{ "top-level domain",			DKIM_PRESULT_TOPLEVEL },
+	{ "parent domain check",		DKIM_PRESULT_PARENTOK },
+	{ "policy test flag set",		DKIM_PRESULT_TESTFLAG },
+	{ "policy \"all\" and valid signature",	DKIM_PRESULT_ALLVALIDSIG },
+	{ "valid originator signature",		DKIM_PRESULT_VALIDOSIG },
+	{ "policy check failed",		DKIM_PRESULT_POLICYERROR },
+	{ NULL,					-1 }
+};
+struct nametable *policyresults = prv_policyresults;
+
 static struct nametable prv_sigparams[] =	/* signature parameters */
 {
 	{ "a",		DKIM_PARAM_SIGNALG },
@@ -146,6 +161,40 @@ static struct nametable prv_settypes[] =	/* set types */
 };
 struct nametable *settypes = prv_settypes;
 
+static struct nametable prv_sigerrors[] =	/* signature parsing errors */
+{
+	{ "no signature error", 		DKIM_SIGERROR_OK },
+	{ "unsupported signature version",	DKIM_SIGERROR_VERSION },
+	{ "invalid domain coverage",		DKIM_SIGERROR_DOMAIN },
+	{ "signature expired",			DKIM_SIGERROR_EXPIRED },
+	{ "signature timestamp in the future",	DKIM_SIGERROR_FUTURE },
+	{ "signature timestamp order error",	DKIM_SIGERROR_TIMESTAMPS },
+	{ "canonicalization missing",		DKIM_SIGERROR_MISSING_C },
+	{ "invalid header canonicalization",	DKIM_SIGERROR_INVALID_HC },
+	{ "invalid body canonicalization",	DKIM_SIGERROR_INVALID_BC },
+	{ "signature algorithm missing",	DKIM_SIGERROR_MISSING_A },
+	{ "signature algorithm invalid",	DKIM_SIGERROR_INVALID_A },
+	{ "header list missing",		DKIM_SIGERROR_MISSING_H },
+	{ "body length value invalid",		DKIM_SIGERROR_INVALID_L },
+	{ "query method invalid",		DKIM_SIGERROR_INVALID_Q },
+	{ "query option invalid",		DKIM_SIGERROR_INVALID_QO },
+	{ "domain tag missing",			DKIM_SIGERROR_MISSING_D },
+	{ "domain tag empty",			DKIM_SIGERROR_EMPTY_D },
+	{ "selector tag missing",		DKIM_SIGERROR_MISSING_S },
+	{ "selector tag empty",			DKIM_SIGERROR_EMPTY_S },
+	{ "signature data missing",		DKIM_SIGERROR_MISSING_B },
+	{ "signature data empty",		DKIM_SIGERROR_EMPTY_B },
+	{ "signature data corrupt",		DKIM_SIGERROR_CORRUPT_B },
+	{ "key not found in DNS",		DKIM_SIGERROR_NOKEY },
+	{ "key DNS reply corrupt",		DKIM_SIGERROR_DNSSYNTAX },
+	{ "key DNS query failed",		DKIM_SIGERROR_KEYFAIL },
+	{ "body hash missing",			DKIM_SIGERROR_MISSING_BH },
+	{ "body hash empty",			DKIM_SIGERROR_EMPTY_BH },
+	{ "body hash corrupt",			DKIM_SIGERROR_CORRUPT_BH },
+	{ NULL,					-1 },
+};
+struct nametable *sigerrors = prv_sigerrors;
+
 /* ===================================================================== */
 
 /*
@@ -168,11 +217,11 @@ dkim_code_to_name(struct nametable *tbl, const int code)
 
 	for (c = 0; ; c++)
 	{
+		if (tbl[c].tbl_code == -1 && tbl[c].tbl_name == NULL)
+			return NULL;
+
 		if (tbl[c].tbl_code == code)
 			return tbl[c].tbl_name;
-
-		if (tbl[c].tbl_code == -1)
-			return NULL;
 	}
 }
 
@@ -196,8 +245,9 @@ dkim_name_to_code(struct nametable *tbl, const char *name)
 
 	for (c = 0; ; c++)
 	{
-		if (tbl[c].tbl_name == NULL)
+		if (tbl[c].tbl_code == -1 && tbl[c].tbl_name == NULL)
 			return -1;
+
 		if (strcasecmp(tbl[c].tbl_name, name) == 0)
 			return tbl[c].tbl_code;
 	}
@@ -224,8 +274,9 @@ dkim_code_to_char(struct chartable *tbl, const int code)
 
 	for (c = 0; ; c++)
 	{
-		if (tbl[c].tbl_code == -1)
+		if (tbl[c].tbl_code == -1 && tbl[c].tbl_char == '\0')
 			return '\0';
+
 		if (tbl[c].tbl_code == code)
 			return tbl[c].tbl_char;
 	}
@@ -252,9 +303,10 @@ dkim_char_to_code(struct chartable *tbl, const int chr)
 
 	for (c = 0; ; c++)
 	{
+		if (tbl[c].tbl_code == -1 && tbl[c].tbl_char == '\0')
+			return -1;
+
 		if (tbl[c].tbl_char == chr)
 			return tbl[c].tbl_code;
-		if (tbl[c].tbl_char == '\0')
-			return -1;
 	}
 }
