@@ -7,7 +7,7 @@
 #define _DKIM_H_
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.106 2007/06/26 16:28:53 msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.129 2007/08/21 12:38:21 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -31,7 +31,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.106 2007/06/26 16:28:53 msk Exp $
 **  	pp == patch number
 */
 
-#define	DKIM_LIB_VERSION	0x01020000
+#define	DKIM_LIB_VERSION	0x02010200
 
 /* the basics */
 #ifndef NULL
@@ -55,7 +55,9 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.106 2007/06/26 16:28:53 msk Exp $
 #endif /* ! MAX */
 
 /* limits, macros, etc. */
+#define	BUFRSZ			1024	/* base temp buffer size */
 #define MAXADDRESS		256	/* biggest user@host we accept */
+#define	MAXBUFRSZ		65536	/* max temp buffer size */
 #define MAXCNAMEDEPTH		3	/* max. CNAME recursion we allow */
 #define MAXHDRCNT		64	/* max. headers signed */
 #define MAXHEADER		4096	/* buffer for caching one header */
@@ -72,8 +74,6 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.106 2007/06/26 16:28:53 msk Exp $
 /* version */
 #define	DKIM_VERSION_KEY	"DKIM1"	/* current version token for keys */
 #define	DKIM_VERSION_SIG	"1"	/* current version token for sigs */
-#define	DKIM_VERSION_SIG_DRAFT1	"0.2"	/* draft version token for sigs */
-#define	DKIM_VERSION_SIG_DRAFT2	"0.5"	/* draft version token for sigs */
 
 /* headers */
 #define	DKIM_FROMHEADER		"From"	/* From: header */
@@ -86,19 +86,6 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.106 2007/06/26 16:28:53 msk Exp $
 					/* reserved DNS sub-zone */
 #define	DKIM_DNSPOLICYNAME	"_ssp"	/* reserved DNS sub-zone */
 
-/*
-**  DKIM_VERSION -- version; attempts to support backward compatibility
-*/
-
-#define DKIM_VERSION_UNKNOWN		0
-#define DKIM_VERSION_UNUSED01		1	/* was ALLMAN_BASE_00 */
-#define DKIM_VERSION_IETF_BASE_00	2
-#define DKIM_VERSION_IETF_BASE_01	3
-#define DKIM_VERSION_IETF_BASE_02	4
-#define DKIM_VERSION_IETF_BASE_10	5
-#define DKIM_VERSION_RFC4871		6
-#define DKIM_VERSION_DEFAULT		DKIM_VERSION_RFC4871
-
 /*
 **  DKIM_STAT -- status code type
 */
@@ -131,6 +118,42 @@ typedef int DKIM_CBSTAT;
 #define	DKIM_CBSTAT_REJECT	1	/* reject */
 #define	DKIM_CBSTAT_TRYAGAIN	2	/* try again later */
 
+/*
+**  DKIM_SIGERROR -- signature errors
+*/
+
+typedef int DKIM_SIGERROR;
+
+#define DKIM_SIGERROR_UNKNOWN		(-1)	/* unknown error */
+#define DKIM_SIGERROR_OK		0	/* no error */
+#define DKIM_SIGERROR_VERSION		1	/* unsupported version */
+#define DKIM_SIGERROR_DOMAIN		2	/* invalid domain (d=/i=) */
+#define DKIM_SIGERROR_EXPIRED		3	/* signature expired */
+#define DKIM_SIGERROR_FUTURE		4	/* signature in the future */
+#define DKIM_SIGERROR_TIMESTAMPS	5	/* x= < t= */
+#define DKIM_SIGERROR_MISSING_C		6	/* c= missing */
+#define DKIM_SIGERROR_INVALID_HC	7	/* c= invalid (header) */
+#define DKIM_SIGERROR_INVALID_BC	8	/* c= invalid (body) */
+#define DKIM_SIGERROR_MISSING_A		9	/* a= missing */
+#define DKIM_SIGERROR_INVALID_A		10	/* a= invalid */
+#define DKIM_SIGERROR_MISSING_H		11	/* h= missing */
+#define DKIM_SIGERROR_INVALID_L		12	/* l= invalid */
+#define DKIM_SIGERROR_INVALID_Q		13	/* q= invalid */
+#define DKIM_SIGERROR_INVALID_QO	14	/* q= option invalid */
+#define DKIM_SIGERROR_MISSING_D		15	/* d= missing */
+#define DKIM_SIGERROR_EMPTY_D		16	/* d= empty */
+#define DKIM_SIGERROR_MISSING_S		17	/* s= missing */
+#define DKIM_SIGERROR_EMPTY_S		18	/* s= empty */
+#define DKIM_SIGERROR_MISSING_B		19	/* b= missing */
+#define DKIM_SIGERROR_EMPTY_B		20	/* b= empty */
+#define DKIM_SIGERROR_CORRUPT_B		21	/* b= corrupt */
+#define DKIM_SIGERROR_NOKEY		22	/* no key found in DNS */
+#define DKIM_SIGERROR_DNSSYNTAX		23	/* DNS reply corrupt */
+#define DKIM_SIGERROR_KEYFAIL		24	/* DNS query failed */
+#define DKIM_SIGERROR_MISSING_BH	25	/* bh= missing */
+#define DKIM_SIGERROR_EMPTY_BH		26	/* bh= empty */
+#define DKIM_SIGERROR_CORRUPT_BH	27	/* bh= corrupt */
+
 /*
 **  DKIM_CANON -- canonicalization method
 */
@@ -225,6 +248,7 @@ typedef int dkim_pparam_t;
 
 typedef int dkim_policy_t;
 
+#define DKIM_POLICY_NONE	(-1)	/* none/undefined */
 #define DKIM_POLICY_UNKNOWN	0	/* unknown */
 #define DKIM_POLICY_ALL		1	/* all */
 #define DKIM_POLICY_STRICT	2	/* strict */
@@ -241,6 +265,21 @@ typedef int dkim_policy_t;
 
 #define DKIM_PFLAG_DEFAULT	0x00
 
+
+/*
+**  DKIM_PRESULT -- policy results
+*/
+
+#define DKIM_PRESULT_NONE		(-1)	/* none/undefined */
+#define DKIM_PRESULT_NXDOMAIN		0	/* domain does not exist */
+#define DKIM_PRESULT_UNKNOWN		1	/* policy is "unknown" */
+#define DKIM_PRESULT_TOPLEVEL		2	/* top-level domain */
+#define DKIM_PRESULT_PARENTOK		3	/* parent domain check OK */
+#define DKIM_PRESULT_TESTFLAG		4	/* policy marked "test" */
+#define DKIM_PRESULT_ALLVALIDSIG	5	/* policy "all", valid sig. */
+#define DKIM_PRESULT_VALIDOSIG		6	/* valid originator sig. */
+#define DKIM_PRESULT_POLICYERROR	7	/* policy check failed */
+
 /*
 **  DKIM_SETTYPE -- types of sets
 */
@@ -288,24 +327,27 @@ typedef int dkim_opts_t;
 #define	DKIM_OPTS_TIMEOUT	2
 #define	DKIM_OPTS_SENDERHDRS	3
 #define	DKIM_OPTS_SIGNHDRS	4
-#define	DKIM_OPTS_VERSION	5
+#define	DKIM_OPTS_VERSION	5	/* unused */
 #define	DKIM_OPTS_QUERYMETHOD	6
 #define	DKIM_OPTS_QUERYINFO	7
 #define	DKIM_OPTS_FIXEDTIME	8
 #define	DKIM_OPTS_SKIPHDRS	9
 #define	DKIM_OPTS_ALWAYSHDRS	10
 #define	DKIM_OPTS_SIGNATURETTL	11
+#define	DKIM_OPTS_CLOCKDRIFT	12
 
-#define	DKIM_LIBFLAGS_NONE	0x00
-#define	DKIM_LIBFLAGS_TMPFILES	0x01
-#define	DKIM_LIBFLAGS_KEEPFILES	0x02
-#define	DKIM_LIBFLAGS_SIGNLEN	0x04
+#define	DKIM_LIBFLAGS_NONE		0x00
+#define	DKIM_LIBFLAGS_TMPFILES		0x01
+#define	DKIM_LIBFLAGS_KEEPFILES		0x02
+#define	DKIM_LIBFLAGS_SIGNLEN		0x04
 #ifdef _FFR_QUERY_CACHE
-# define DKIM_LIBFLAGS_CACHE	0x10
+# define DKIM_LIBFLAGS_CACHE		0x08
 #endif /* _FFR_QUERY_CACHE */
-#define DKIM_LIBFLAGS_ZTAGS	0x20
+#define DKIM_LIBFLAGS_ZTAGS		0x10
+#define DKIM_LIBFLAGS_DELAYSIGPROC	0x20
+#define DKIM_LIBFLAGS_EOHCHECK		0x40
 
-#define	DKIM_LIBFLAGS_DEFAULT	DKIM_LIBFLAGS_NONE
+#define	DKIM_LIBFLAGS_DEFAULT		DKIM_LIBFLAGS_NONE
 
 /*
 **  DKIM_LIB -- library handle
@@ -362,6 +404,13 @@ typedef struct dkim_key DKIM_KEY;
 struct dkim_policy;
 typedef struct dkim_policy DKIM_POLICY;
 
+/*
+**  DKIM_CANON -- canonicalization
+*/
+
+struct dkim_canon;
+typedef struct dkim_canon DKIM_CANON;
+
 /*
 **  DKIM_SIGINFO -- signature information for use by the caller
 */
@@ -372,11 +421,24 @@ typedef struct dkim_siginfo DKIM_SIGINFO;
 #define DKIM_SIGFLAG_IGNORE	0x01
 #define DKIM_SIGFLAG_PROCESSED	0x02
 #define DKIM_SIGFLAG_PASSED	0x04
+#define DKIM_SIGFLAG_TESTKEY	0x08
 
 #define DKIM_SIGBH_UNTESTED	(-1)
 #define DKIM_SIGBH_MATCH	0
 #define DKIM_SIGBH_MISMATCH	1
 
+#ifdef _FFR_DIFFHEADERS
+/*
+**  DKIM_HDRDIFF -- header differences
+*/
+
+struct dkim_hdrdiff
+{
+	u_char *		hd_old;
+	u_char *		hd_new;
+};
+#endif /* _FFR_DIFFHEADERS */
+
 /*
 **  PROTOTYPES
 */
@@ -517,6 +579,34 @@ extern DKIM_STAT dkim_body __P((DKIM *dkim, u_char *buf, size_t len));
 
 extern DKIM_STAT dkim_eom __P((DKIM *dkim, bool *testkey));
 
+/*
+**  DKIM_GETSIGLIST -- retrieve the list of signatures
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**   	sigs -- pointer to a vector of DKIM_SIGINFO pointers (updated)
+**   	nsigs -- pointer to an integer to receive the pointer count (updated)
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_getsiglist __P((DKIM *dkim, DKIM_SIGINFO ***sigs,
+                                      int *nsigs));
+
+/*
+**  DKIM_GETSIGNATURE -- retrieve the "final" signature
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**
+**  Return value:
+**  	Pointer to a DKIM_SIGINFO handle which is the one libdkim will
+**  	use to return a "final" result; NULL if none could be determined.
+*/
+
+extern DKIM_SIGINFO *dkim_getsignature __P((DKIM *dkim));
+
 /*
 **  DKIM_GETSIGHDR -- compute and return a signature header for a message
 **
@@ -535,14 +625,32 @@ extern DKIM_STAT dkim_getsighdr __P((DKIM *dkim, u_char *buf, size_t len,
                                      size_t margin, size_t initial));
 
 /*
-**  DKIM_GETCANONLEN -- report number of (canonicalized) body bytes that were
-**                      signed
+**  DKIM_SIG_GETIDENTITY -- retrieve identity of the signer
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle (or NULL to choose final one)
+**  	val -- destination buffer
+**  	vallen -- size of destination buffer
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_sig_getidentity __P((DKIM *dkim, DKIM_SIGINFO *sig,
+                                           char *val, size_t vallen));
+
+/*
+**  DKIM_SIG_GETCANONLEN -- report number of (canonicalized) body bytes that
+**                          were signed
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_sign() or
 **  	        dkim_verify()
+**  	sig -- a DKIM_SIGINFO handle
 **  	msglen -- total size of the message body (returned)
 **  	canonlen -- total number of canonicalized bytes (returned)
+**  	signlen -- restricted signature length (returned)
 **
 **  Return value:
 **  	A DKIM_STAT value.
@@ -552,14 +660,16 @@ extern DKIM_STAT dkim_getsighdr __P((DKIM *dkim, u_char *buf, size_t len,
 **  	to the caller.
 */
 
-extern DKIM_STAT dkim_getcanonlen __P((DKIM *dkim, off_t *msglen,
-                                       off_t *canonlen));
+extern DKIM_STAT dkim_sig_getcanonlen __P((DKIM *dkim, DKIM_SIGINFO *sig,
+                                           off_t *msglen, off_t *canonlen,
+                                           off_t *signlen));
 
 /*
 **  DKIM_REPORTINFO -- return info needed to generate failure report
 **
 **  Parameters:
 **  	dkim -- a DKIM handle previously returned by dkim_verify()
+**  	sig -- a DKIM_SIGINFO handle previously returned by dkim_getsignature()
 **  	fd -- pointer to receive temporary file descriptor
 **  	bfd -- pointer to receive temporary body file descriptor
 **   	raddr -- buffer to receive report address
@@ -569,7 +679,8 @@ extern DKIM_STAT dkim_getcanonlen __P((DKIM *dkim, off_t *msglen,
 **  	A DKIM_STAT value.
 */
 
-extern DKIM_STAT dkim_reportinfo __P((DKIM *dkim, int *fd, int *bfd,
+extern DKIM_STAT dkim_reportinfo __P((DKIM *dkim, DKIM_SIGINFO *sig,
+                                      int *fd, int *bfd,
                                       u_char *raddr, size_t rlen));
 
 /*
@@ -590,107 +701,108 @@ extern DKIM_STAT dkim_options __P((DKIM_LIB *dklib, int op, dkim_opts_t opt,
                                    void *ptr, size_t len));
 
 /*
-**  DKIM_POLICY -- retreive signing policy associated with the sender's domain
+**  DKIM_SIG_GETFLAGS -- retreive signature handle flags
 **
 **  Parameters:
-**  	dkim -- DKIM handle
-**  	test -- test flag (returned)
+**  	sig -- DKIM_SIGINFO handle
 **
 **  Return value:
-**  	A DKIM_POLICY_* value, or -1 on error.
+**  	An unsigned integer which is a bitwise-OR of the DKIM_SIGFLAG_*
+**  	constants currently set in the provided handle.
 */
 
-extern dkim_policy_t dkim_policy __P((DKIM *dkim, bool *test));
+extern unsigned int dkim_sig_getflags __P((DKIM_SIGINFO *sig));
 
 /*
-**  DKIM_GETKEYSIZE -- retreive key size after verifying
+**  DKIM_SIG_GETBH -- retreive signature handle "bh" test state
 **
 **  Parameters:
-**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
+**
+**  Return value:
+**  	An unsigned integer which is one of the DKIM_SIGBH_* constants
+**  	indicating the current state of "bh" evaluation of the signature.
+*/
+
+extern unsigned int dkim_sig_getbh __P((DKIM_SIGINFO *sig));
+
+/*
+**  DKIM_SIG_GETKEYSIZE -- retreive key size after verifying
+**
+**  Parameters:
+**  	sig -- DKIM_SIGINFO handle
 **  	bits -- size of the key in bits (returned)
 **
 **  Return value:
 **  	A DKIM_STAT value.
 */
 
-extern DKIM_STAT dkim_getkeysize __P((DKIM *dkim, unsigned int *bits));
+extern DKIM_STAT dkim_sig_getkeysize __P((DKIM_SIGINFO *sig,
+                                          unsigned int *bits));
 
 /*
-**  DKIM_GETSIGNALG -- retreive signature algorithm after verifying
+**  DKIM_SIG_GETSIGNALG -- retreive signature algorithm after verifying
 **
 **  Parameters:
-**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
 **  	alg -- a DKIM_SIGN_* value (returned)
 **
 **  Return value:
 **  	A DKIM_STAT value.
 */
 
-extern DKIM_STAT dkim_getsignalg __P((DKIM *dkim, dkim_alg_t *alg));
+extern DKIM_STAT dkim_sig_getsignalg __P((DKIM_SIGINFO *sig, dkim_alg_t *alg));
 
 /*
-**  DKIM_GETSIGNTIME -- retreive signature timestamp after verifying
+**  DKIM_SIG_GETSIGNTIME -- retreive signature timestamp after verifying
 **
 **  Parameters:
-**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
 **  	when -- timestamp on the signature (returned)
 **
 **  Return value:
 **  	A DKIM_STAT value.
 */
 
-extern DKIM_STAT dkim_getsigntime __P((DKIM *dkim, time_t *when));
+extern DKIM_STAT dkim_sig_getsigntime __P((DKIM_SIGINFO *sig, time_t *when));
 
 /*
-**  DKIM_GETSELECTOR -- retrieve selector used to generate the signature
+**  DKIM_SIG_GETSELECTOR -- retrieve selector used to generate the signature
 **
 **  Parameters:
-**  	dkim -- DKIM handle from which to retrieve selector
+**  	sig -- DKIM_SIGINFO handle from which to retrieve selector
 **
 **  Return value:
 **  	Selector found in the signature.
 */
 
-extern const char * dkim_getselector __P((DKIM *dkim));
+extern char *dkim_sig_getselector __P((DKIM_SIGINFO *sig));
 
 /*
-**  DKIM_GETSIGDOMAIN -- retrieve signing domain after verifying
+**  DKIM_SIG_GETDOMAIN -- retrieve signing domain after verifying
 **
 **  Parameters:
-**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
 **
 **  Return value:
 **  	Pointer to the signing domain.
 */
 
-extern const char *dkim_getsigdomain __P((DKIM *dkim));
+extern char *dkim_sig_getdomain __P((DKIM_SIGINFO *sig));
 
 #ifdef _FFR_STATS
 /*
-**  DKIM_GETCANONS -- retrieve canonicaliztions after verifying
+**  DKIM_SIG_GETCANONS -- retrieve canonicaliztions after verifying
 **
 **  Parameters:
-**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
 **
 **  Return value:
 **  	Pointer to the signing domain.
 */
 
-extern DKIM_STAT dkim_getcanons __P((DKIM *dkim, dkim_canon_t *hdr,
-                                     dkim_canon_t *body));
-
-/*
-**  DKIM_GETVERSION -- retreive signing version after verifying
-**
-**  Parameters:
-**  	dkim -- DKIM handle
-**  	ver -- signature version (returned)
-**
-**  Return value:
-**  	A DKIM_STAT value.
-*/
-
-extern DKIM_STAT dkim_getversion __P((DKIM *dkim, int *ver));
+extern DKIM_STAT dkim_sig_getcanons __P((DKIM_SIGINFO *sig, dkim_canon_t *hdr,
+                                         dkim_canon_t *body));
 #endif /* _FFR_STATS */
 
 /*
@@ -706,6 +818,18 @@ extern DKIM_STAT dkim_getversion __P((DKIM *dkim, int *ver));
 
 extern DKIM_STAT dkim_set_user_context __P((DKIM *dkim, const void *ctx));
 
+/*
+**  DKIM_GET_USER_CONTEXT -- retrieve DKIM handle user context
+**
+**  Parameters:
+**  	dkim -- DKIM signing handle
+**
+**  Parameters:
+**  	User context pointer.
+*/
+
+extern const void *dkim_get_user_context __P((DKIM *dkim));
+
 /*
 **  DKIM_SET_SIGNER -- set DKIM signature's signer
 **
@@ -747,9 +871,10 @@ extern DKIM_STAT dkim_set_dns_callback __P((DKIM_LIB *libdkim,
 */
 
 extern DKIM_STAT dkim_set_key_lookup __P((DKIM_LIB *libdkim,
-                                          DKIM_STAT (*func)(DKIM *dkim,
-                                                            u_char *buf,
-                                                            size_t buflen)));
+                                          DKIM_CBSTAT (*func)(DKIM *dkim,
+                                                              DKIM_SIGINFO *sig,
+                                                              u_char *buf,
+                                                              size_t buflen)));
 
 /*
 **  DKIM_SET_POLICY_LOOKUP -- set the policy lookup function
@@ -763,9 +888,11 @@ extern DKIM_STAT dkim_set_key_lookup __P((DKIM_LIB *libdkim,
 */
 
 extern DKIM_STAT dkim_set_policy_lookup __P((DKIM_LIB *libdkim,
-                                             int (*func)(DKIM *dkim,
-                                                         u_char *buf,
-                                                         size_t buflen)));
+                                             DKIM_CBSTAT (*func)(DKIM *dkim,
+                                                                 u_char *query,
+                                                                 u_char *buf,
+                                                                 size_t buflen,
+                                                                 int *qstat)));
 
 /*
 **  DKIM_SET_SIGNATURE_HANDLE -- set the signature handle creator function
@@ -826,9 +953,26 @@ extern DKIM_STAT dkim_set_signature_tagvalues __P((DKIM_LIB *libdkim,
 */
 
 extern DKIM_STAT dkim_set_prescreen __P((DKIM_LIB *libdkim,
-                                         DKIM_CBSTAT (*func)(DKIM_SIGINFO **sigs,
+                                         DKIM_CBSTAT (*func)(DKIM *dkim,
+                                                             DKIM_SIGINFO **sigs,
                                                              int nsigs)));
 
+/*
+**  DKIM_SET_FINAL -- set the final processing function
+**
+**  Parameters:
+**  	libdkim -- DKIM library handle
+**  	func -- function to call
+**
+**  Return value:
+**  	DKIM_STAT_OK
+*/
+
+extern DKIM_STAT dkim_set_final __P((DKIM_LIB *libdkim,
+                                     DKIM_CBSTAT (*func)(DKIM *dkim,
+                                                         DKIM_SIGINFO **sigs,
+                                                         int nsigs)));
+
 /*
 **  DKIM_SIG_GETCONTEXT -- get user-specific context from a DKIM_SIGINFO
 **
@@ -842,6 +986,31 @@ extern DKIM_STAT dkim_set_prescreen __P((DKIM_LIB *libdkim,
 
 extern void *dkim_sig_getcontext __P((DKIM_SIGINFO *siginfo));
 
+/*
+**  DKIM_SIG_GETERROR -- get error code from a DKIM_SIGINFO
+**
+**  Parameters:
+**  	siginfo -- a pointer to a DKIM_SIGINFO
+**
+**  Return value:
+**  	A DKIM_SIGERROR_* constant.
+*/
+
+extern int dkim_sig_geterror __P((DKIM_SIGINFO *siginfo));
+
+/*
+**  DKIM_SIG_GETERRORSTR -- translate a DKIM_SIGERROR into a string
+**
+**  Parameters:
+**  	sigerr -- a DKIM_SIGERROR constant
+**
+**  Return value:
+**  	A pointer to a human-readable string translation of "sigerr", or NULL
+**  	if no such translation exists.
+*/
+
+extern const char *dkim_sig_geterrorstr __P((DKIM_SIGERROR sigerr));
+
 /*
 **  DKIM_SIG_IGNORE -- mark a signature referenced by a DKIM_SIGINFO with
 **                     an "ignore" flag
@@ -855,6 +1024,35 @@ extern void *dkim_sig_getcontext __P((DKIM_SIGINFO *siginfo));
 
 extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
 
+/*
+**  DKIM_POLICY -- parse policy associated with the sender's domain
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	test -- policy test flag (returned)
+**  	susp -- suspicious message flag (returned)
+**  	pcode -- discovered policy (returned)
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_policy __P((DKIM *dkim, bool *test, bool *susp,
+                                  dkim_policy_t *pcode));
+
+/*
+**  DKIM_SIG_PROCESS -- process a signature
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+*/
+
+extern DKIM_STAT dkim_sig_process __P((DKIM *dkim, DKIM_SIGINFO *sig));
+
 /*
 **  DKIM_FREE -- release resources associated with a DKIM handle
 **
@@ -881,11 +1079,36 @@ extern DKIM_STAT dkim_free __P((DKIM *dkim));
 
 extern const char *dkim_geterror __P((DKIM *dkim));
 
+/*
+**  DKIM_GETPRESULT -- retrieve policy result
+**
+**  Parameters:
+**  	dkim -- DKIM handle from which to get policy result
+**
+**  Return value:
+**  	DKIM policy check result.
+*/
+
+extern int dkim_getpresult __P((DKIM *dkim));
+
+/*
+**  DKIM_GETPRESULTSTR -- retrieve policy result string
+**
+**  Parameters:
+**  	presult -- policy result code to translate
+**
+**  Return value:
+**  	Pointer to text that describes "presult".
+*/
+
+extern const char *dkim_getpresultstr __P((int presult));
+
 /*
 **  DKIM_OHDRS -- extract and decode original headers
 **
 **  Parameters:
 **  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
 **  	ptrs -- user-provided array of pointers to header strings (updated)
 **  	pcnt -- number of pointers available (updated)
 **
@@ -893,7 +1116,33 @@ extern const char *dkim_geterror __P((DKIM *dkim));
 **  	A DKIM_STAT_* constant.
 */
 
-extern DKIM_STAT dkim_ohdrs __P((DKIM *dkim, char **ptrs, int *pcnt));
+extern DKIM_STAT dkim_ohdrs __P((DKIM *dkim, DKIM_SIGINFO *sig, char **ptrs,
+                                 int *pcnt));
+
+#ifdef _FFR_DIFFHEADERS
+/*
+**  DKIM_DIFFHEADERS -- compare original headers with received headers
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	maxcost -- maximum "cost" of changes to be reported
+**  	ohdrs -- original headers, presumably extracted from a "z" tag
+**  	nohdrs -- number of headers at "ohdrs" available
+**  	out -- pointer to an array of struct dkim_hdrdiff objects (updated)
+** 	nout -- counter of handles returned (updated)
+**
+**  Return value:
+**  	A DKIM_STAT_* constant.
+**
+**  Side effects:
+**  	A series of DKIM_HDRDIFF handles is allocated and must later be
+**  	destroyed.
+*/
+
+extern DKIM_STAT dkim_diffheaders __P((DKIM *dkim, int maxcost, char **ohdrs,
+                                       int nohdrs, struct dkim_hdrdiff **out,
+                                       int *nout));
+#endif /* _FFR_DIFFHEADERS */
 
 /*
 **  RFC2822_MAILBOX_SPLIT -- extract the userid and host from a structured
